// File: parameters.cc
//
// Time-stamp: <2008-07-18 19:11:00 haog>
// Author: Guohua Hao <haog@eecs.oregonstate.edu>
//
// Copyright (C) 2008 Guohua Hao
// All Rights Reserved.
//
// Description: implementation of Parameters class
//
//////////////////////////////////////////////////////////////////////

#include <cstdlib>
#include <fstream>
#include <iostream>
#include "parameters.h"

namespace treecrf {

Parameters::Parameters() {
  iterations = 10;
  maxLeaves = 30;
  npad = 1;
  testInterval = 1;
  saveInterval = 1;
  filestem = "predictions-";
  fbWidth = 10;
  viterbiWidth = 10;
  predictRSW = 0;
  predictFB = 1;
  printTime = 1;
  sampleRate = 1;
  scale = 1;
  trim = 0;
  trim_val = 0.01;
  treeFeatureRate = 1;
  iterCutoff = 0;
  initMaxLeaves = 0;
  shrinkage = 0;
  maxSetSize = 5000;
  printViterbi = 0;
  printFB = 0;
  normalize = 1;
  betaPass = 1;
  sum_fb = 1;
}

Parameters::Parameters(char * filename) {
  Read(filename);
}

void Parameters::skipComments(std::istream & str) {
  char c;
  while (1) {
    str >> c;
    if (c == '/') {
      // this introduces a comment. Skip to end of line
      while (c != '\n') str.get(c);
    }
    else {
      str.putback(c);
      return;
    }
  }
}

void Parameters::Read(char * filename) {
  std::ifstream str(filename);
  if (!str) {
    std::cerr << "Can't open parameter file " << filename << std::endl;
    std::abort();
  }
  skipComments(str);
  str >> iterations;
  skipComments(str);
  str >> maxLeaves;
  skipComments(str);
  str >> npad;
  skipComments(str);
  str >> testInterval;
  skipComments(str);
  str >> saveInterval;
  skipComments(str);
  str >> filestem;
  skipComments(str);
  str >> fbWidth;
  skipComments(str);
  str >> viterbiWidth;
  skipComments(str);
  str >> predictRSW;
  skipComments(str);
  str >> predictFB;
  skipComments(str);
  str >> printTime;
  skipComments(str);
  str >> sampleRate;
  skipComments(str);
  str >> scale;
  skipComments(str);
  str >> trim;
  skipComments(str);
  str >> trim_val;
  skipComments(str);
  str >> treeFeatureRate;
  skipComments(str);
  str >> iterCutoff;
  skipComments(str);
  str >> initMaxLeaves;
  skipComments(str);
  str >> shrinkage;
  skipComments(str);
  str >> maxSetSize;
  skipComments(str);
  str >> printViterbi;
  skipComments(str);
  str >> printFB;
  skipComments(str);
  str >> normalize;
  skipComments(str);
  str >> betaPass;
  skipComments(str);
  str >> sum_fb;
  skipComments(str);
}

void Parameters::Display(std::ostream & str) {
  using namespace std;

  str << "Critical Parameters" << endl;
  str << "iterations \t" << iterations << endl;
  str << "maxLeaves\t" << maxLeaves << endl;
  str << "npad\t" << npad << endl;
  str << "testInterval\t" << testInterval << endl;
  str << "saveInterval\t" << saveInterval << endl;
  str << "filestem\t" << filestem << endl;
  str << "fbWidth\t" << fbWidth << endl;
  str << "viterbiWidth\t" << viterbiWidth << endl;
  str << "predictRSW\t" << predictRSW << endl;
  str << "predictFB\t" << predictFB << endl;
  str << "printTime\t" << printTime << endl;

  str << "\nParameters for avoiding overfitting" << endl;
  str << "sampleRate\t" << sampleRate << endl;
  str << "scale\t" << scale << endl;
  str << "trim\t" << trim << endl;
  str << "trim_val\t" << trim_val << endl;
  str << "treeFeatureRate\t" << treeFeatureRate << endl;
  str << "iterCutoff\t" << iterCutoff << endl;
  str << "initMaxLeaves\t" << initMaxLeaves << endl;
  str << "shrinkage\t" << shrinkage << endl;

  str << "\nOther parameters..." << endl;
  str << "maxSetSize\t" << maxSetSize << endl;
  str << "printViterbi\t" << printViterbi << endl;
  str << "printFB\t" << printFB << endl;
  str << "normalize\t" << normalize << endl;
  str << "betaPass\t" << betaPass << endl;
  str << "sum_fb\t" << sum_fb << endl;
}

} // namespace treecrf

//////////////////////////////////////////////////////////////////////
