// File: potential_function.h
//
// Time-stamp: <2008-07-18 10:47:17 haog>
// Author: Guohua Hao <haog@eecs.oregonstate.edu>
//
// Copyright (C) 2008 Guohua Hao
// All Rights Reserved.
//
// Description: Regression Tree Potential Function
//
//////////////////////////////////////////////////////////////////////

#ifndef _POTENTIAL_FUNCTION_H_
#define _POTENTIAL_FUNCTION_H_

#include <iostream>
#include <list>
#include <vector>

namespace treecrf {

class DataSet;
class Example;
class RegressionTree;

// A regression tree potential function is a list of regression trees
class RTPotentialFunction {
 public:
  // Constructors
  RTPotentialFunction(): cache_(0), generation_(0), current_generation_(0) {};
  explicit RTPotentialFunction(int cache_size):
    cache_(cache_size, 0.0), generation_(cache_size, -2), current_generation_(0) {};

  // Must release the pointers to regression trees
  ~RTPotentialFunction();

  // Compute potential score (un exponentiated) for an example
  double Score(const Example & ex); // caching version
  double RawScore(const Example & ex) const; // non-caching version

  // Train another tree for the potential function based on the given
  // dataset and max num of leaves.
  void Train(const DataSet & dataset, int num_leaves);

  // IO
  void Print(std::ostream & str) const;
  void Read(std::istream & str);

 private:
  // Data fields
  // cached values of the potential function
  std::vector<double> cache_;
  // last valid generation of the cached values
  std::vector<int> generation_;
  // number of current generation
  int current_generation_;
  std::list<RegressionTree *> trees_;
};

void operator << (std::ostream & str, const RTPotentialFunction & pf);
void operator >> (std::istream & str, RTPotentialFunction & pf);

} // namespace treecrf

#endif  // _POTENTIAL_FUNCTION_H_
//////////////////////////////////////////////////////////////////////
