// File: rsw.cc
//
// Time-stamp: <2008-07-17 18:55:41 haog>
// Author: Guohua Hao <haog@eecs.oregonstate.edu>
//
// Copyright (C) 2008 Guohua Hao
// All Rights Reserved.
//
// Description: implementation of RswEval class
//
//////////////////////////////////////////////////////////////////////

#include <algorithm>
#include "dataset.h"
#include "rsw.h"
#include "treecrf.h"

namespace treecrf {

void RswEval::Predict(const DataSet & dataset,
                      const TreeCRF & pfc,
                      int* predict_classes) {
  int nwindows = dataset.GetNumExamples();
  int num_features = dataset.num_features();
  int num_classes = dataset.num_classes() + 1; // including the off-the-end class

  int loc = dataset.GetExample(0)->GetSize() - k_val_;
  int offset = num_features - (num_classes * k_val_);

  // std::cout << "START SEQ!!!!!!!!!!!!!!!!" << std::endl;
  for (int x = 0; x < nwindows; ++x) {
    Example* ex = new Example(*dataset.GetExample(x));

    // define features based on the last prediction.
    // recurrent_feature will be the insertion point for the new features.
    // k_val_ = npad = order of the markov model.
    int recurrent_feature = offset;
    int current_loc = loc;
    int upper_bound = std::min(x, k_val_);
    // y = relative offset to yth previous prediction
    for (int y = 1; y <= upper_bound; ++y) {
      ex->SetFeatureAt(current_loc, recurrent_feature + predict_classes[x-y] + 1);
      current_loc++;
      recurrent_feature += num_classes;
    }
    predict_classes[x] = pfc.ClassifyExample(*ex);
    delete ex;
  }
}

} // namespace treecrf

//////////////////////////////////////////////////////////////////////
